import Ember from 'ember';
var isEmpty = Ember.isEmpty;
var typeOf = Ember.typeOf;

export default Ember.Component.extend({
	// Component Properties
	tagName: 'div',
	classNames: ['ui-bs-button-choice','btn-group'],
	attributeBindings: ['dataToggle:data-toggle'],
	dataToggle: 'buttons',
	// Data Properties
	choices: null,
	labelProp: 'name', // the property name to display when given a hash
	valueProp: 'id', // the property name to use as a reference key when given a hash
	type: 'radio', // radio/checkbox
	missingChoicesText: 'no choices available',
	canBeNull: null,
	_canBeNull: function() {
		Ember.run.next(this, function() {
			if(this.get('canBeNull') === null) {
				this.set('canBeNull', this.get('type') === 'radio' ? false : true);
			}
		});
	}.on('didInsertElement'),
	color: 'default', 
	colorClass: function() {
		return 'btn-%@'.fmt(this.get('color'));
	}.property('color'),
	size: 'default',
	sizeClass: function() {
		var size = this.get('size');
		if(['lg','large'].contains(size)) {
			return 'btn-lg';
		} else if(['sm','small'].contains(size)) {
			return 'btn-sm';
		} else if(['xs','tiny'].contains(size)) {
			return 'btn-xs';
		} else {
			return false;
		}
	}.property('size'),
	selected: null,
	selectedObject: null,
	choicesHash: [],
	_choicesHash: function() {
		Ember.run.next(this,function() {
			var p = this.getProperties('choices','labelProp','valueProp','type','selected');
			if(Ember.typeOf(p.choices) === 'string') {
				p.choices = p.choices.split(',');
			}
			if (isEmpty(p.choices)) {
				return [];
			}
			var choicesHash = p.choices.map(function(item, index) {
				var hash; 
				if(Ember.typeOf(item) === 'string') {
					hash = {};
					hash.id = item;
					hash.name = item;
				} else {
					// copy object
					hash = JSON.parse(JSON.stringify(item));
					// ensure interface is met
					hash.id = item[p.valueProp];
					hash.name = item[p.labelProp];
				}
				if (
					(Ember.typeOf(p.selected) === 'array' && p.selected.contains(hash.id)) ||
					(Ember.typeOf(p.selected) === 'string' && p.selected === hash.id)
				) {
					hash.checked = true;
				}
				return hash;
			});
			if(isEmpty(choicesHash.filterBy('checked',true)) && !this.get('canBeNull')) {
				choicesHash[0].checked = true;
				this.set('selected',choicesHash[0].id);
			}

			this.set('choicesHash', choicesHash);
		});
	}.observes('choices','choices.length','label','value','type','selected').on('didInsertElement'),
	click: function(evt) {
		Ember.run.next(this, function() {
			var action = this.processEvent();
			if(action.hasChanged) {
				this.set('selected', action.selected);
				this.set('selectedObject', action.selectedObject);
				this.sendAction('changeEvent', action);
			} else if(this.get('canBeNull')) {
				this.set('selected', null);
				this.set('selectedObject', null);
			}
		});
	},
	processEvent: function() {
		var selected;
		var selectedObject;
		var self = this;
		if (this.get('type') === 'radio') {
			selected = this.$('.active input').val();
			selectedObject = this.get('choicesHash').findBy('id',selected);
		} else {
			selected = [];
			selectedObject = [];
			this.$('.active input').each(function() {
				var newItem = $(this).val();
				selected.pushObject(newItem);
				selectedObject.pushObject(self.get('choicesHash').findBy('id',newItem));
			});
		}
		// determine the change (if any)
		var componentState = this.getProperties('selected','selectedObject');
		var type = this.get('type');
		var state = {
			isEmpty: null,
			howMany: null,
			hasChanged: null, // maybe be overridden below
			changeSource: null, // [external,bootstrap]
			added: [], // list of ID(s) just added 
			removed: [] // list of ID(s) just removed 
		};
		// radio versus checkbox
		if(type === 'radio') {
			// existance
			if(isEmpty(selected)) {
				state.isEmpty = true;
				state.howMany = 0;
			} else {
				state.isEmpty = false;
				state.howMany = 1;
			}
			// changes
			if(componentState.selected !== selected ) {
				state.hasChanged = true;
				state.changeSource = 'bootstrap';
				if(!state.isEmpty) {
					state.added = [ selected ];
				}
				if(!Ember.isEmpty(componentState.selected)) {
					state.removed = [ componentState.selected ];
				}
			} else {
				// if change was done directly to "selected" bound property
				// we will end up in this state but there will be delta in selectedObject
				if(isEmpty(componentState.selectedObject) || componentState.selectedObject.id !== selectedObject.id) {
					state.hasChanged = true;
					state.changeSource = 'external';	
					if(!isEmpty(selectedObject)) {
						state.added = [ selectedObject.id ];						
					}
					if(!isEmpty(componentState.selectedObject)) {
						state.removed = [ componentState.selectedObject.id ];						
					}					
				} else {
					state.hasChanged = false;
				}
			}
		} else {
			// existance
			if(Ember.isEmpty(selected)) {
				state.isEmpty = true;
				state.howMany = 0;
			} else {
				state.isEmpty = false;
				state.howMany = selected.length;
			}
			// changes
			componentState.selected = isEmpty(componentState.selected) ? [] : componentState.selected;
			if(typeOf(componentState.selected) === 'string') {
				// a checkbox's should always be a selector but if its a string then convert gracefully and warn to console
				componentState.selected = componentState.selected.split(',');
			}
			selected = selected || [];
			if(componentState.selected.join(',') !== selected.join(',')) {
				state.hasChanged = true;
				state.changeSource = 'bootstrap';
				componentState.selected.forEach(function(item,index) {
					if (!selected.contains(item)) {
						state.removed.addObject(item);
					}
				});
				selected.forEach(function(item,index) {
					if (!componentState.selected.contains(item)) {
						state.added.addObject(item);
					}
				});
			} else {
				// if change was done directly to "selected" bound property
				// we will end up in this state but there will be delta in selectedObject
				componentState.selectedObject = componentState.selectedObject || [];
				if(componentState.selectedObject.mapBy('id').join(',') !== selectedObject.mapBy('id').join(',')) {
					state.hasChanged = true;
					state.changeSource = 'external';	
					if(!isEmpty(selectedObject)) {
						selectedObject.mapBy('id').forEach(function(item,index) {
							if(!componentState.selectedObject.mapBy('id').contains(item)) {
								state.added.addObject(item);
							}
						});				
						componentState.selectedObject.mapBy('id').forEach(function(item,index) {
							if(!selectedObject.mapBy('id').contains(item)) {
								state.removed.addObject(item);
							}
						});				
					}
					if(!isEmpty(componentState.selectedObject)) {
						state.removed = [ componentState.selectedObject.id ];						
					}					
				} else {
					state.hasChanged = false;
				}
			}
		}
		// handle no selections if not allowed
		if(!this.get('canBeNull') && state.isEmpty) {
			// TODO: implement
			console.log('property that can not be null, IS.');
		}
		state.selected = selected;
		state.selectedObject = selectedObject;
		return state;
	},
	selectedObserver: function() {
		Ember.run.next(this, function() {
			var action = this.processEvent();
			if(action.hasChanged) {
				this.set('selectedObject', action.selectedObject);
			} 
		});
	}.observes('selected')
});
